@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python3
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
@@ -22,11 +22,9 @@
 GTK graphical user interface
 """
 
-from __future__ import absolute_import, print_function
-
 from bleachbit import GuiBasic
 from bleachbit import Cleaner, FileUtilities
-from bleachbit import _, APP_NAME, appicon_path, portable_mode
+from bleachbit import _, APP_NAME, appicon_path, portable_mode, windows10_theme_path
 from bleachbit.Options import options
 from bleachbit.GuiPreferences import PreferencesDialog
 from bleachbit.Cleaner import backends, register_cleaners
@@ -39,7 +37,6 @@ import os
 import sys
 import threading
 import time
-import types
 
 import gi
 gi.require_version('Gtk', '3.0')
@@ -59,6 +56,60 @@ def threaded(func):
     return wrapper
 
 
+def notify_gi(msg):
+    """Show a pop-up notification.
+
+    The Windows pygy-aio installer does not include notify, so this is just for Linux.
+    """
+    gi.require_version('Notify', '0.7')
+    from gi.repository import Notify
+    if Notify.init(APP_NAME):
+        notify = Notify.Notification.new('BleachBit', msg, 'bleachbit')
+        notify.set_hint("desktop-entry", GLib.Variant('s', 'bleachbit'))
+        notify.show()
+        notify.set_timeout(10000)
+
+
+def notify_plyer(msg):
+    """Show a pop-up notification.
+
+    Linux distributions do not include plyer, so this is just for Windows.
+    """
+    from bleachbit import bleachbit_exe_path
+
+    # On Windows 10,  PNG does not work.
+    __icon_fns = (
+        os.path.normpath(os.path.join(bleachbit_exe_path,
+                                      'share\\bleachbit.ico')),
+        os.path.normpath(os.path.join(bleachbit_exe_path,
+                                      'windows\\bleachbit.ico')))
+
+    icon_fn = None
+    for __icon_fn in __icon_fns:
+        if os.path.exists(__icon_fn):
+            icon_fn = __icon_fn
+            break
+
+    from plyer import notification
+    notification.notify(
+        title=APP_NAME,
+        message=msg,
+        app_name=APP_NAME,  # not shown on Windows 10
+        app_icon=icon_fn,
+    )
+
+
+def notify(msg):
+    """Show a popup-notification"""
+    import importlib
+    if importlib.util.find_spec('plyer'):
+        # On Windows, use Plyer.
+        notify_plyer(msg)
+        return
+    # On Linux, use GTK Notify.
+    notify_gi(msg)
+
+
 class Bleachbit(Gtk.Application):
     _window = None
     _shred_paths = None
@@ -96,6 +147,7 @@ class Bleachbit(Gtk.Application):
 
         On Windows with GTK 3.18, this cde is sufficient for the menu to work.
         """
+
         builder = Gtk.Builder()
         builder.add_from_file(bleachbit.app_menu_filename)
         menu = builder.get_object('app-menu')
@@ -212,8 +264,7 @@ class Bleachbit(Gtk.Application):
         self._window.preview_or_run_operations(True, operations)
 
     def get_preferences_dialog(self):
-        return PreferencesDialog(
-            self._window, self._window.cb_refresh_operations)
+        return self._window.get_preferences_dialog()
 
     def cb_preferences_dialog(self, action, param):
         """Callback for preferences dialog"""
@@ -273,14 +324,14 @@ class Bleachbit(Gtk.Application):
         textview.set_editable(False)
         swindow = Gtk.ScrolledWindow()
         swindow.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
-        swindow.add_with_viewport(textview)
+        swindow.add(textview)
         dialog.vbox.pack_start(swindow, True, True, 0)
         dialog.add_buttons(Gtk.STOCK_COPY, 100,
                            Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE)
-        return dialog
+        return (dialog, txt)
 
     def diagnostic_dialog(self, _action, _param):
-        dialog = self.get_diagnostics_dialog()
+        dialog, txt = self.get_diagnostics_dialog()
         dialog.show_all()
         while True:
             rc = dialog.run()
@@ -407,7 +458,7 @@ class TreeDisplayModel:
 
     def set_cleaner(self, path, model, parent_window, value):
         """Activate or deactive option of cleaner."""
-        assert isinstance(value, types.BooleanType)
+        assert isinstance(value, bool)
         assert isinstance(model, Gtk.TreeStore)
         cleaner_id = None
         i = path
@@ -427,8 +478,8 @@ class TreeDisplayModel:
             # %(option) may be cache, logs, cookies, etc.
             # %(warning) may be 'This option is really slow'
             msg = _("Warning regarding %(cleaner)s - %(option)s:\n\n%(warning)s") % \
-                {'cleaner': model[parent][0].decode('utf-8'),
-                 'option': model[path][0].decode('utf-8'),
+                {'cleaner': model[parent][0],
+                 'option': model[path][0],
                  'warning': warning}
             if warning:
                 resp = GuiBasic.message_dialog(parent_window,
@@ -468,6 +519,9 @@ class TreeDisplayModel:
 
 class GUI(Gtk.ApplicationWindow):
     """The main application GUI"""
+    _style_provider = None
+    _style_provider_regular = None
+    _style_provider_dark = None
 
     def __init__(self, auto_exit, *args, **kwargs):
         super(GUI, self).__init__(*args, **kwargs)
@@ -491,6 +545,7 @@ class GUI(Gtk.ApplicationWindow):
             # if stderr was redirected - keep redirecting it
             sys.stderr = self.gtklog
 
+        self.set_windows10_theme()
         Gtk.Settings.get_default().set_property(
             'gtk-application-prefer-dark-theme', options.get('dark_mode'))
 
@@ -499,38 +554,14 @@ class GUI(Gtk.ApplicationWindow):
                 _('Resetting the configuration file because it is corrupt: %s') % bleachbit.options_file)
             bleachbit.Options.init_configuration()
 
-        if options.get("first_start") and not auto_exit:
-            if os.name == 'posix':
-                self.append_text(
-                    _('Access the application menu by clicking the hamburger icon on the title bar.'))
-                pref = PreferencesDialog(self, self.cb_refresh_operations)
-                pref.run()
-            if os.name == 'nt':
-                self.append_text(
-                    _('Access the application menu by clicking the logo on the title bar.'))
-            options.set('first_start', False)
-        if os.name == 'nt':
-            # BitDefender false positive.  BitDefender didn't mark BleachBit as infected or show
-            # anything in its log, but sqlite would fail to import unless BitDefender was in "game mode."
-            # http://bleachbit.sourceforge.net/forum/074-fails-errors
-            try:
-                import sqlite3
-            except ImportError as e:
-                self.append_text(
-                    _("Error loading the SQLite module: the antivirus software may be blocking it."), 'error')
-
-        if os.name == 'posix' and bleachbit.expanduser('~') == '/root':
-            self.append_text(
-                _('You are running BleachBit with administrative privileges for cleaning shared parts of the system, and references to the user profile folder will clean only the root account.'))
-        if os.name == 'nt' and options.get('shred'):
-            from win32com.shell.shell import IsUserAnAdmin
-            if not IsUserAnAdmin():
-                self.append_text(
-                    _('Run BleachBit with administrator privileges to improve the accuracy of overwriting the contents of files.'))
-                self.append_text('\n')
-
         GLib.idle_add(self.cb_refresh_operations)
 
+    def get_preferences_dialog(self):
+        return PreferencesDialog(
+            self,
+            self.cb_refresh_operations,
+            self.set_windows10_theme)
+
     def shred_paths(self, paths, shred_settings=False, quit_when_done=False):
         """Shred file or folders
 
@@ -683,7 +714,7 @@ class GUI(Gtk.ApplicationWindow):
         else:
             self.start_time = time.time()
             worker = self.worker.run()
-            GLib.idle_add(worker.next)
+            GLib.idle_add(worker.__next__, priority=GLib.PRIORITY_LOW)
 
     def worker_done(self, worker, really_delete):
         """Callback for when Worker is done"""
@@ -705,21 +736,7 @@ class GUI(Gtk.ApplicationWindow):
         logger.debug('elapsed time: %d seconds', elapsed)
         if elapsed < 10 or self.is_active():
             return
-        try:
-            gi.require_version('Notify', '0.7')
-            from gi.repository import Notify
-        except:
-            logger.debug('Notify not available')
-        else:
-            if Notify.init(APP_NAME):
-                notify = Notify.Notification.new(
-                    'BleachBit', _("Done."), 'bleachbit')
-                if os.name == 'posix' and bleachbit.expanduser('~') == '/root':
-                    notify.set_hint("desktop-entry", "bleachbit-root")
-                else:
-                    notify.set_hint("desktop-entry", "bleachbit")
-                notify.show()
-                notify.set_timeout(10000)
+        notify(_("Done."))
 
     def create_operations_box(self):
         """Create and return the operations box (which holds a tree view)"""
@@ -747,7 +764,7 @@ class GUI(Gtk.ApplicationWindow):
         self.progressbar.show()
         rc = register_cleaners(self.update_progress_bar,
                                self.cb_register_cleaners_done)
-        GLib.idle_add(rc.next)
+        GLib.idle_add(rc.__next__)
         return False
 
     def cb_register_cleaners_done(self):
@@ -759,11 +776,47 @@ class GUI(Gtk.ApplicationWindow):
         self.view.expand_all()
 
         # Check for online updates.
-        if bleachbit.online_update_notification_enabled and options.get("check_online_updates") and \
+        if not self.auto_exit and \
+            bleachbit.online_update_notification_enabled and \
+            options.get("check_online_updates") and \
                 not hasattr(self, 'checked_for_updates'):
             self.checked_for_updates = True
             self.check_online_updates()
 
+        # Show information for first start.
+        # (The first start flag is set also for each new version.)
+        if options.get("first_start") and not self.auto_exit:
+            if os.name == 'posix':
+                self.append_text(
+                    _('Access the application menu by clicking the hamburger icon on the title bar.'))
+                pref = self.get_preferences_dialog()
+                pref.run()
+            if os.name == 'nt':
+                self.append_text(
+                    _('Access the application menu by clicking the logo on the title bar.'))
+            options.set('first_start', False)
+
+        if os.name == 'nt':
+            # BitDefender false positive.  BitDefender didn't mark BleachBit as infected or show
+            # anything in its log, but sqlite would fail to import unless BitDefender was in "game mode."
+            # http://bleachbit.sourceforge.net/forum/074-fails-errors
+            try:
+                import sqlite3
+            except ImportError as e:
+                self.append_text(
+                    _("Error loading the SQLite module: the antivirus software may be blocking it."), 'error')
+
+        # Show notice about admin privileges.
+        if os.name == 'posix' and os.path.expanduser('~') == '/root':
+            self.append_text(
+                _('You are running BleachBit with administrative privileges for cleaning shared parts of the system, and references to the user profile folder will clean only the root account.')+'\n')
+        if os.name == 'nt' and options.get('shred'):
+            from win32com.shell.shell import IsUserAnAdmin
+            if not IsUserAnAdmin():
+                self.append_text(
+                    _('Run BleachBit with administrator privileges to improve the accuracy of overwriting the contents of files.'))
+                self.append_text('\n')
+
         # remove from idle loop (see GObject.idle_add)
         return False
 
@@ -843,7 +896,7 @@ class GUI(Gtk.ApplicationWindow):
         """Callback to update the progress bar with number or text"""
         if isinstance(status, float):
             self.progressbar.set_fraction(status)
-        elif isinstance(status, (str, unicode)):
+        elif isinstance(status, str):
             self.progressbar.set_show_text(True)
             self.progressbar.set_text(status)
         else:
@@ -899,7 +952,8 @@ class GUI(Gtk.ApplicationWindow):
             icon_size = Gtk.IconSize.LARGE_TOOLBAR
 
         # create the preview button
-        self.preview_button = Gtk.Button.new_from_icon_name('edit-find', icon_size)
+        self.preview_button = Gtk.Button.new_from_icon_name(
+            'edit-find', icon_size)
         self.preview_button.set_always_show_image(True)
         self.preview_button.connect(
             'clicked', lambda *dummy: self.preview_or_run_operations(False))
@@ -911,7 +965,8 @@ class GUI(Gtk.ApplicationWindow):
         box.add(self.preview_button)
 
         # create the delete button
-        self.run_button = Gtk.Button.new_from_icon_name('edit-clear-all', icon_size)
+        self.run_button = Gtk.Button.new_from_icon_name(
+            'edit-clear-all', icon_size)
         self.run_button.set_always_show_image(True)
         # TRANSLATORS: This is the clean button on the main window.
         # It makes permanent changes: usually deleting files, sometimes
@@ -923,7 +978,8 @@ class GUI(Gtk.ApplicationWindow):
         box.add(self.run_button)
 
         # stop cleaning
-        self.stop_button = Gtk.Button.new_from_icon_name('process-stop', icon_size)
+        self.stop_button = Gtk.Button.new_from_icon_name(
+            'process-stop', icon_size)
         self.stop_button.set_always_show_image(True)
         self.stop_button.set_label(_('Abort'))
         self.stop_button.set_tooltip_text(
@@ -951,21 +1007,24 @@ class GUI(Gtk.ApplicationWindow):
         return hbar
 
     def on_configure_event(self, widget, event):
-        # save window position and size
-        screen = self.get_screen()
-        (screen_w, screen_h) = (screen.get_width(), screen.get_height())
+        # fixup maximized window position:
+        # on Windows if a window is maximized on a secondary monitor it is moved off the screen
         (x, y) = self.get_position()
-        if x >= screen_w or y >= screen_h:
-            monitor = screen.get_monitor_at_window(self.get_window())
-            g = screen.get_monitor_geometry(monitor)
-            print("Fixup moving the window off the screen: window (x, y) = {}, screen (w, h) = {}, monitor (x, y) = {}, (w, h) = {}".format(
-                (x, y), (screen_w, screen_h),
-                (g.x, g.y), (g.width, g.height)))
-            self.move(g.x, g.y)
-            return True
+        (width, height) = self.get_size()
+        window = self.get_window()
+        if window.get_state() & Gdk.WindowState.MAXIMIZED != 0:
+            screen = self.get_screen()
+            monitor_num = screen.get_monitor_at_window(window)
+            g = screen.get_monitor_geometry(monitor_num)
+            if x < g.x or x >= g.x + g.width or y < g.y or y >= g.y + g.height:
+                logger.info("Maximized window {}+{}: monitor ({}) geometry = {}+{}".format(
+                    (x, y), (width, height), monitor_num, (g.x, g.y), (g.width, g.height)))
+                self.move(g.x, g.y)
+                return True
+
+        # save window position and size
         options.set("window_x", x, commit=False)
         options.set("window_y", y, commit=False)
-        (width, height) = self.get_size()
         options.set("window_width", width, commit=False)
         options.set("window_height", height, commit=False)
         return False
@@ -985,16 +1044,58 @@ class GUI(Gtk.ApplicationWindow):
 
     def on_show(self, widget):
         # restore window position, size and state
-        if options.has_option("window_x") and options.has_option("window_y"):
-            self.move(options.get("window_x"), options.get("window_y"))
-        if options.has_option("window_width") and options.has_option("window_height"):
-            self.resize(options.get("window_width"),
-                        options.get("window_height"))
+        if options.has_option("window_x") and options.has_option("window_y") and \
+           options.has_option("window_width") and options.has_option("window_height"):
+            r = Gdk.Rectangle()
+            (r.x, r.y) = (options.get("window_x"), options.get("window_y"))
+            (r.width, r.height) = (options.get(
+                "window_width"), options.get("window_height"))
+
+            screen = self.get_screen()
+            monitor_num = screen.get_monitor_at_point(r.x, r.y)
+            g = screen.get_monitor_geometry(monitor_num)
+
+            # only restore position and size if window left corner
+            # is within the closest monitor
+            if r.x >= g.x and r.x < g.x + g.width and \
+               r.y >= g.y and r.y < g.y + g.height:
+                logger.info("closest monitor ({}) geometry = {}+{}, window geometry = {}+{}".format(
+                    monitor_num, (g.x, g.y), (g.width, g.height), (r.x, r.y), (r.width, r.height)))
+                self.move(r.x, r.y)
+                self.resize(r.width, r.height)
         if options.get("window_fullscreen"):
             self.fullscreen()
         elif options.get("window_maximized"):
             self.maximize()
 
+    def set_windows10_theme(self):
+        """Toggle the Windows 10 theme"""
+        if not 'nt' == os.name:
+            return
+
+        if not self._style_provider_regular:
+            self._style_provider_regular = Gtk.CssProvider()
+            self._style_provider_regular.load_from_path(
+                os.path.join(windows10_theme_path, 'gtk.css'))
+        if not self._style_provider_dark:
+            self._style_provider_dark = Gtk.CssProvider()
+            self._style_provider_dark.load_from_path(
+                os.path.join(windows10_theme_path, 'gtk-dark.css'))
+
+        screen = Gdk.Display.get_default_screen(Gdk.Display.get_default())
+        if self._style_provider is not None:
+            Gtk.StyleContext.remove_provider_for_screen(
+                screen, self._style_provider)
+        if options.get("win10_theme"):
+            if options.get("dark_mode"):
+                self._style_provider = self._style_provider_dark
+            else:
+                self._style_provider = self._style_provider_regular
+            Gtk.StyleContext.add_provider_for_screen(
+                screen, self._style_provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
+        else:
+            self._style_provider = None
+
     def populate_window(self):
         """Create the main application window"""
         screen = self.get_screen()
