@@ -22,8 +22,6 @@
 Logging
 """
 
-from __future__ import absolute_import
-
 import logging
 
 
@@ -41,7 +39,7 @@ class DelayLog(object):
     def read(self):
         for msg in self.queue:
             yield msg
-        queue = []
+        self.queue = []
 
     def write(self, msg):
         self.msg += msg
