@@ -22,8 +22,6 @@
 Store and retrieve user preferences
 """
 
-from __future__ import absolute_import
-
 import bleachbit
 from bleachbit import General
 from bleachbit import _
@@ -50,9 +48,10 @@ boolean_keys = ['auto_hide',
                 'shred',
                 'units_iec',
                 'window_maximized',
-                'window_fullscreen', ]
+                'window_fullscreen']
 if 'nt' == os.name:
     boolean_keys.append('update_winapp2')
+    boolean_keys.append('win10_theme')
 int_keys = ['window_x', 'window_y', 'window_width', 'window_height', ]
 
 
@@ -74,7 +73,7 @@ def init_configuration():
     if os.path.lexists(bleachbit.options_file):
         logger.debug('Deleting configuration: %s ' % bleachbit.options_file)
         os.remove(bleachbit.options_file)
-    with open(bleachbit.options_file, 'w') as f_ini:
+    with open(bleachbit.options_file, 'w', encoding='utf-8-sig') as f_ini:
         f_ini.write('[bleachbit]\n')
         if os.name == 'nt' and bleachbit.portable_mode:
             f_ini.write('[Portable]\n')
@@ -91,8 +90,8 @@ class Options:
         self.purged = False
         self.config = bleachbit.RawConfigParser()
         self.config.optionxform = str  # make keys case sensitive for hashpath purging
-        self.config._boolean_states['t'] = True
-        self.config._boolean_states['f'] = False
+        self.config.BOOLEAN_STATES['t'] = True
+        self.config.BOOLEAN_STATES['f'] = False
         self.restore()
 
     def __flush(self):
@@ -102,16 +101,16 @@ class Options:
         if not os.path.exists(bleachbit.options_dir):
             General.makedirs(bleachbit.options_dir)
         mkfile = not os.path.exists(bleachbit.options_file)
-        _file = open(bleachbit.options_file, 'wb')
-        try:
-            self.config.write(_file)
-        except IOError as e:
-            from errno import ENOSPC
-            if e.errno == ENOSPC:
-                logger.error(
-                    _("Disk was full when writing configuration to file %s"), bleachbit.options_file)
-            else:
-                raise
+        with open(bleachbit.options_file, 'w', encoding='utf-8-sig') as _file:
+            try:
+                self.config.write(_file)
+            except IOError as e:
+                from errno import ENOSPC
+                if e.errno == ENOSPC:
+                    logger.error(
+                        _("Disk was full when writing configuration to file %s"), bleachbit.options_file)
+                else:
+                    raise
         if mkfile and General.sudo_mode():
             General.chownself(bleachbit.options_file)
 
@@ -126,7 +125,6 @@ class Options:
                 # restore colon lost because ConfigParser treats colon special
                 # in keys
                 pathname = pathname[0] + ':' + pathname[1:]
-                pathname = pathname.decode('utf-8')
             exists = False
             try:
                 exists = os.path.lexists(pathname)
@@ -162,7 +160,7 @@ class Options:
             return self.config.getboolean(section, option)
         elif option in int_keys:
             return self.config.getint(section, option)
-        return self.config.get(section, option.encode('utf-8'))
+        return self.config.get(section, option)
 
     def get_hashpath(self, pathname):
         """Recall the hash for a file"""
@@ -250,7 +248,7 @@ class Options:
     def restore(self):
         """Restore saved options from disk"""
         try:
-            self.config.read(bleachbit.options_file)
+            self.config.read(bleachbit.options_file, encoding='utf-8-sig')
         except:
             logger.exception("Error reading application's configuration")
         if not self.config.has_section("bleachbit"):
@@ -280,6 +278,7 @@ class Options:
 
         if 'nt' == os.name:
             self.__set_default("update_winapp2", False)
+            self.__set_default("win10_theme", False)
 
         if not self.config.has_section('preserve_languages'):
             lang = bleachbit.user_locale
@@ -287,7 +286,7 @@ class Options:
             if -1 != pos:
                 lang = lang[0: pos]
             for _lang in set([lang, 'en']):
-                logger.info(_("Automatically preserving language %s."), lang)
+                logger.info(_("Automatically preserving language %s."), _lang)
                 self.set_language(_lang, True)
 
         # BleachBit upgrade or first start ever
@@ -300,7 +299,7 @@ class Options:
 
     def set(self, key, value, section='bleachbit', commit=True):
         """Set a general option"""
-        self.config.set(section, key.encode('utf-8'), str(value))
+        self.config.set(section, key, str(value))
         if commit:
             self.__flush()
 
